//********************************************************************************
//*    lib_mot_moway_10.h                                                        *
//*    version: 1.0                                                              *
//*    Esta librera contiene las funciones necesarias para gestionar el mdulo  *
//*    motor del robot Moway con programas del compilador CCS                    *
//*    Copyright (C) 2007  Bizintek Innova S.L.                                  *
//********************************************************************************
//*    This program is free software; you can redistribute it and/or modify      *
//*    it under the terms of the GNU General Public License as published by      *
//*    the Free Software Foundation; either version 2 of the License, or         *
//*    (at your option) any later version.                                       *
//*                                                                              *
//*    This program is distributed in the hope that it will be useful,           *
//*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
//*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
//*    GNU General Public License for more details.                              *
//*                                                                              *
//*    You should have received a copy of the GNU General Public License along   *
//*    with this program; if not, write to the Free Software Foundation, Inc.,   *
//*    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.               *
//********************************************************************************

//Elimina los warning condicin siempre verdadera
#ignore_warnings 203

//********************
//*   DEFINICIONES   *
//********************

//Configuracin I2C
#use i2c(Master,Fast,sda=PIN_C1,scl=PIN_C0,restart_wdt)

//*****************************
//*Configuracion del patillaje*
//*****************************

// PORTC
#define SDA           PIN_C1
#define SCL           PIN_C0

// PORTB
#define MOT_END       PIN_B5

// TRISC
#define SCL_TRIS      TRISC,0
#define SDA_TRIS      TRISC,1

// TRISB
#define MOT_END_TRIS  TRISB,5

//*****************
//*Variables fijas*
//*****************

//Direccin del mdulo motor

#define   MOT_DIR         0x10

//Comandos
//Byte MOT_COMAND                nbit    7   6   5   4   3   2   1   0
//                                       --------------------------------
//Comandos movimientos
#define   COM_STR         0x00   //      "0   0   0   0   0   0   CT  RL"
#define   COM_CHA_VEL     0x01   //      "0   0   0   0   0   1   CT  RL"
#define   COM_ROT         0x02   //      "0   0   0   0   1   0   CT  RL"
#define   COM_CUR         0x03   //      "0   0   0   0   1   1   CT  RL"
#define   COM_STOP        0x04   //      "0   0   0   1   0   0   CT  RL"

//Comandos informacin
#define   STATUS_T        0x00   //      "1   0   0   0   0   0   0   0 "
#define   STATUS_A        0x01   //      "1   0   0   0   0   0   0   1 "
#define   STATUS_V_R      0x02   //      "1   0   0   0   0   0   1   0 "
#define   STATUS_V_L      0x03   //      "1   0   0   0   0   0   1   1 "
#define   STATUS_D_R      0x04   //      "1   0   0   0   0   1   0   0 "
#define   STATUS_D_L      0x05   //      "1   0   0   0   0   1   0   1 "
#define   STATUS_KM       0x06   //      "1   0   0   0   0   1   1   0 "

//Comandos reset
#define   RST_T           0x00   //      "1   1   1   1   0   0   0   0 "
#define   RST_D           0x01   //      "1   1   1   1   0   0   0   1 "
#define   RST_KM          0x02   //      "1   1   1   1   0   0   1   0 "


//Parmetros
#define   RIGHT           1
#define   LEFT            0

#define   DISTANCE        0
#define   TIME            1
#define   ANGLE           0

#define   FWD             1
#define   BACK            0

#define   CENTER          1
#define   WHEEL           0


//*****************
//*   VARIABLES   *
//*****************
#BYTE TRISA  =  0x85
#BYTE TRISB  =  0x86
#BYTE TRISC  =  0x87

//Variables internas
static int8  MOT_COMAND;
static int1  MOT_ACK;
static int1  MOT_SND_OK;
static int8  MOT_STATUS_DATA_0;
static int8  MOT_STATUS_DATA_1;
static int8  MOT_RE_SND;


//**************
//*   CDIGO   *
//**************



//*****************************************************
//*              MOT_SND_I2C_1D(int8 )                *
//*****************************************************
//*Descripcin:Funcin de comunicacin por I2C de bajo*
//*nivel encargado de mandar un solo dato.            *
//*****************************************************
//*Variables de entrada:- Dato a mandar               *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*****************************************************

int1 MOT_SND_I2C_1D(int8 MOT_DATA_0)
{
   MOT_RE_SND=10;

   while(MOT_RE_SND>0){
      while(1){
      i2c_start();
      MOT_ACK=i2c_write(MOT_DIR);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_0);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      i2c_stop();
      MOT_RE_SND=0;
      MOT_SND_OK=0;
      break;
      }
   }
   return(MOT_SND_OK);
}
//*****************************************************










//*****************************************************
//*       MOT_SND_I2C_3D(int8, int8, int8)            *
//*****************************************************
//*Descripcin:Funcin de comunicacin por I2C de bajo*
//*nivel encargado de mandar tres datos.              *
//*****************************************************
//*Variables de entrada:- Datos a mandar              *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*****************************************************
int1 MOT_SND_I2C_3D(int8 MOT_DATA_0,int8 MOT_DATA_1,int8 MOT_DATA_2)
{
   MOT_RE_SND=10;

   while(MOT_RE_SND>0){
      while(1){
      i2c_start();
      MOT_ACK=i2c_write(MOT_DIR);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_0);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_1);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_2);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      i2c_stop();
      MOT_RE_SND=0;
      MOT_SND_OK=0;
      break;
      }
   }
   return(MOT_SND_OK);
}
//*****************************************************







//*****************************************************
//*     MOT_SND_I2C_4D(int8, int8, int8, int8)        *
//*****************************************************
//*Descripcin:Funcin de comunicacin por I2C de bajo*
//*nivel encargado de mandar cuatro datos.            *
//*****************************************************
//*Variables de entrada:- Datos a mandar              *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*****************************************************
int1 MOT_SND_I2C_4D(int8 MOT_DATA_0,int8 MOT_DATA_1,int8 MOT_DATA_2, int8 MOT_DATA_3)
{
   MOT_RE_SND=10;

   while(MOT_RE_SND>0){
      while(1){
      i2c_start();
      MOT_ACK=i2c_write(MOT_DIR);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_COMAND);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_1);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      MOT_ACK=i2c_write(MOT_DATA_2);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
      }
      MOT_ACK=i2c_write(MOT_DATA_3);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
         }
      i2c_stop();
      MOT_RE_SND=0;
      MOT_SND_OK=0;
      break;
      }
   }
   return(MOT_SND_OK);
}
//*****************************************************










//*****************************************************
//*                   MOT_CONFIG()                    *
//*****************************************************
//*Descripcin:Esta funcin se encarga de configurar  *
//*el PIC para que puede comunicarse con el mdulo    *
//*motor                                              *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida:                               *
//*****************************************************
void MOT_CONFIG()
{
   bit_set(SCL_TRIS);
   bit_set(SDA_TRIS);
   bit_set(MOT_END_TRIS);
}
//*****************************************************








//*****************************************************
//*        MOT_STR(int, int1, int1, int)              *
//*****************************************************
//*Descripcin: Comando para ir recto. Hay que        *
//*especificar velocidad, sentido, tipo de comando y  *
//*el tiempo o la distancia a recorrer                *
//*****************************************************
//*Variables de entrada:- MOT_VEL    : 0    - 100     *
//*                     - FWDBACK    : FWD  - BACK    *
//*                     - COMTYPE    : TIME - DISTANCE*
//*                     - MOT_T_DIST : 0    - 255     *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*                     - 2: Datos incorrectos        *
//*****************************************************
int8 MOT_STR(int MOT_VEL,int1 FWDBACK,int1 COMTYPE, int MOT_T_DIST)
{
   //Comprobacin de datos correctos

   //Velocidad debe estar entre 0 y 100
   if(MOT_VEL>100)
   return(2);

   //Preparacin de datos para ser mandados por I2C

   //El comando a mandar es COM_STR (0x00)
   MOT_COMAND=COM_STR;

   //En LSB de velocidad se empaquetara el sentido
   shift_left (&MOT_VEL, 1, FWDBACK);

   //Se mandan COMTYPE y RL en el mismo byte de MOT_COMAND
   shift_left (&MOT_COMAND, 1, COMTYPE);
   shift_left (&MOT_COMAND, 1, 0);

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_3D(MOT_COMAND,MOT_VEL,MOT_T_DIST));
}
//*****************************************************










//*****************************************************
//*     MOT_CHA_VEL(int, int1, int1, int1, int)       *
//*****************************************************
//*Descripcin: Comando para cambiar la velocidad de  *
//*uno de los motores. Para ello hay que especificar  *
//*velocidad, sentido, motor izquierdo o derecho,tipo *
//*de comando y el tiempo o la distancia a recorrer.  *
//*****************************************************
//*Variables de entrada:- MOT_VEL    : 0    - 100     *
//*                     - FWDBACK    : FWD  - BACK    *
//*                     - RL         : RIGHT- LEFT    *
//*                     - COMTYPE    : TIME - DISTANCE*
//*                     - MOT_T_DIST : 0    - 255     *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*                     - 2: Datos incorrectos        *
//*****************************************************
  int8 MOT_CHA_VEL(int MOT_VEL,int1 FWDBACK,int1 RL,int1 COMTYPE,int MOT_T_DIST)
{
   //Comprobacin de datos correctos

   //Velocidad debe estar entre 0 y 100
   if(MOT_VEL>100)
   return(2);

   //Preparacin de datos para ser mandados por I2C

   //El comando a mandar es COM_CHA_VEL (0x01)
   MOT_COMAND=COM_CHA_VEL;

   //En LSB de velocidad se empaquetara el sentido
   shift_left (&MOT_VEL, 1, FWDBACK);

   //Se mandan COMTYPE y RL en el mismo byte de MOT_COMAND
   shift_left (&MOT_COMAND, 1, COMTYPE);
   shift_left (&MOT_COMAND, 1, RL);

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_3D(MOT_COMAND,MOT_VEL,MOT_T_DIST));
}
//*****************************************************









//*****************************************************
//*     MOT_ROT(int, int1, int, int1, int1, int)      *
//*****************************************************
//*Descripcin: Comando para rotar. Para ello hay que *
//*especificar velocidad, sentido, rotacin           *
//*respecto al centro o a la rueda,tipo de comando y  *
//*el tiempo o el angulo a recorrer                   *
//*****************************************************
//*Variables de entrada:- MOT_VEL     : 0     - 100   *
//*                     - FWDBACK     : FWD   - BACK  *
//*                     - MOT_CENWHEEL: CENTER- WHEEL *
//*                     - RL          : RIGHT - LEFT  *
//*                     - COMTYPE     : TIME  - ANGLE *
//*                     - MOT_T_ANG   :(0-255)-(0-100)*
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*                     - 2: Datos incorrectos        *
//*****************************************************
int8 MOT_ROT(int MOT_VEL,int1 FWDBACK,int MOT_CENWHEEL,int1 RL,int1 COMTYPE,int MOT_T_ANG)
{
   //Comprobacin de datos correctos

   //Velocidad debe estar entre 0 y 100
   if(MOT_VEL>100)
   return(2);

   //Rotacin sobre el centro o sobre una rueda
   if( MOT_CENWHEEL!=1 && MOT_CENWHEEL!=0)
   return(2);

   //Si es un comando angulo este debe ser de 0-100
   //Si no lo es devulve 2 (datos incorrectos)
   if( COMTYPE==0 && MOT_T_ANG>100)
   return(2);

   //Preparacin de datos para ser mandados por I2C

   //El comando a mandar es COM_ROT (0x02)
   MOT_COMAND=COM_ROT;

   //En LSB de velocidad se empaquetara el sentido
   shift_left (&MOT_VEL,1,FWDBACK);

   //Se mandan COMTYPE y RL en el mismo byte de MOT_COMAND
   shift_left (&MOT_COMAND, 1, COMTYPE);
   shift_left (&MOT_COMAND, 1, RL);

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_4D(MOT_COMAND,MOT_VEL,MOT_CENWHEEL,MOT_T_ANG));
}
//*****************************************************







//*****************************************************
//*     MOT_CUR(int, int1, int, int1, int1, int)      *
//*****************************************************
//*Descripcin: Comando para dar curvas. Para ello hay*
//*que especificar velocidad, sentido, radio,         *
//*izquierda o derecha, tipo de comando y el tiempo o *
//*la distancia a recorrer.                           *
//*****************************************************
//*Variables de entrada:- MOT_VEL : 0     - 100       *
//*                     - FWDBACK : FWD   - BACK      *
//*                     - MOT_RAD :                   *
//*                        - 0<(MOT_VEL-MOT_RAD)<100  *
//*                     - RL      : RIGHT- LEFT       *
//*                     - COMTYPE : TIME - DISTANCE   *
//*                     - MOT_T_DIST  : 0    - 255    *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*                     - 2: Datos incorrectos        *
//*****************************************************
int8 MOT_CUR(int MOT_VEL,int1 FWDBACK,int MOT_RAD,int1 RL,int1 COMTYPE, int MOT_T_DIST)
{

   //Comprobacin de datos correctos

   //Velocidad debe estar entre 0 y 100
   if(MOT_VEL>100)
   return(2);

   //Velocidad - radio debe ser menor que 100 y menor que 0
   if((MOT_VEL+MOT_RAD)>100 || (MOT_VEL-MOT_RAD)>156)
   return(2);

   //Preparacin de datos para ser mandados por I2C

   //El comando a mandar es COM_CUR (0x03)
   MOT_COMAND=COM_CUR;

   //En LSB de velocidad se empaquetara el sentido
   shift_left (&MOT_VEL, 1, FWDBACK);

   //Se manda COMTYPE y RL en el mismo byte de MOT_COMAND
   shift_left (&MOT_COMAND, 1, COMTYPE);
   shift_left (&MOT_COMAND, 1, RL);

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_4D(MOT_COMAND,MOT_VEL,MOT_RAD,MOT_T_DIST));
}
//*****************************************************






//*****************************************************
//*                    MOT_STOP()                     *
//*****************************************************
//*Descripcin: Comando para parar.                   *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*                     - 2: Datos incorrectos        *
//*****************************************************
int1 MOT_STOP()
{

   //Preparacin de datos para ser mandados por I2C

   //El comando a mandar es COM_STOP (0x04)
   MOT_COMAND=COM_STOP;

   //Se manda COMTYPE y RL en el mismo byte de MOT_COMAND
   shift_left (&MOT_COMAND, 1, 0);
   shift_left (&MOT_COMAND, 1, 0);

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_1D(MOT_COMAND));
}
//*****************************************************







//*****************************************************
//*                  MOT_RST(int8)                    *
//*****************************************************
//*Descripcin: Comando para resetear tiempo,         *
//*distancia o el cuentakilometros.                   *
//*****************************************************
//*Variables de entrada:- RST_COM:                    *
//*                         -RST_T                    *
//*                         -RST_D                    *
//*                         -RST_KM                   *
//*Variables de salida:                               *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*****************************************************
int1 MOT_RST(int8 RST_COM)
{
   //Adecuacin de los datos a mandar
   RST_COM = RST_COM | 0xF0;

   //Se mandan los datos por I2C
   return(MOT_SND_I2C_1D(RST_COM));
}
//*****************************************************







//*****************************************************
//*                 MOT_FDBCK(int8)                  *
//*****************************************************
//*Descripcin: Comando para conocer varios parmetros*
//*del mdulo motor: tiempo, angulo recorrido,        *
//*velocidad de los dos motores, distancia recorrida  *
//*por los dos motores y valor del cuentakilometros   *
//*(2 variables de salida).                           *
//*****************************************************
//*Variables de entrada:- STATUS_COM:                 *
//*                         -STATUS_T                 *
//*                         -STATUS_A                 *
//*                         -STATUS_V_R               *
//*                         -STATUS_V_L               *
//*                         -STATUS_D_R               *
//*                         -STATUS_D_L               *
//*                         -STATUS_KM                *
//*Variables de salida: - MOT_STATUS_DATA_0           *
//*                     - MOT_STATUS_DATA_1           *
//*Salida:              - 0: Envo correcto           *
//*                     - 1: Envio incorrecto         *
//*****************************************************
int1 MOT_FDBCK(int8 STATUS_COM)
{
      //Adecuacin de los datos a mandar
      bit_set(STATUS_COM,7);

      //Se mandan los datos por I2C
      if(MOT_SND_I2C_1D(STATUS_COM)!=0)
      return(1);

      //Se manda comando de leer
      MOT_RE_SND=10;
      while(MOT_RE_SND>0){
      while(1){
      i2c_start();
      MOT_ACK=i2c_write(0x11);
      if (MOT_ACK){
         i2c_stop();
         MOT_RE_SND--;
         MOT_SND_OK=1;
         break;
      }

      //Se lee el primer dato
      MOT_STATUS_DATA_0=i2c_read(1);
      //Se lee el segundo dato dato
      MOT_STATUS_DATA_1=i2c_read(0);

      i2c_stop();
      MOT_RE_SND=0;
      MOT_SND_OK=0;
      break;
      }
   }
   return(MOT_SND_OK);
}
//*****************************************************
