//********************************************************************************
//*    lib_sen_moway_10.h                                                         *
//*    version: 1.0                                                              *
//*    Esta librera contiene las funciones necesarias para gestionar los        *
//*    sensores del robot Moway                                                  *
//*    Copyright (C) 2007  Bizintek Innova S.L.                                  *
//*                                                                              *
//*   NOTA:Es necesario definir el reloj y el n de bit de ADC antes de          *
//*   incluir las librerias. Se recomienda el siguiente cdigo:                  *
//*                                                                              *
//*   #device adc=8                                                              *
//*   #use delay(clock=4000000)                                                  *
//*                                                                              *
//********************************************************************************
//*    This program is free software; you can redistribute it and/or modify      *
//*    it under the terms of the GNU General Public License as published by      *
//*    the Free Software Foundation; either version 2 of the License, or         *
//*    (at your option) any later version.                                       *
//*                                                                              *
//*    This program is distributed in the hope that it will be useful,           *
//*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
//*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
//*    GNU General Public License for more details.                              *
//*                                                                              *
//*    You should have received a copy of the GNU General Public License along   *
//*    with this program; if not, write to the Free Software Foundation, Inc.,   *
//*    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.               *
//********************************************************************************

//********************
//*   DEFINICIONES   *
//********************

// PORTA
#define   LIGHT_SEN            PIN_A0
#define   IR_RX_R              PIN_A1
#define   LINE_RX_R            PIN_A2
#define   IR_RX_L              PIN_A3
#define   LED_TOP_RED          PIN_A4
#define   LINE_RX_L            PIN_A5

// PORTB
#define   LINE_TX              PIN_B1
#define   IR_TX                PIN_B2
#define   LED_R                PIN_B4
#define   LED_TOP_GREEN        PIN_B6

// PORTC
#define   LED_L                PIN_C6


// TRISA
#define   LIGHT_SEN_TRIS       TRISA,0
#define   IR_RX_R_TRIS         TRISA,1
#define   LINE_RX_R_TRIS       TRISA,2
#define   IR_RX_L_TRIS         TRISA,3
#define   LED_TOP_RED_TRIS     TRISA,4
#define   LINE_RX_L_TRIS       TRISA,5

// TRISB
#define   LINE_TX_TRIS         TRISB,1
#define   IR_TX_TRIS           TRISB,2
#define   LED_R_TRIS           TRISB,4
#define   LED_TOP_GREEN_TRIS   TRISB,6

// TRISC
#define   LED_L_TRIS           TRISC,6


//*****************
//*   VARIABLES   *
//*****************
//Definicin registros PIC
#BYTE TRISA  =  0x85
#BYTE TRISB  =  0x86
#BYTE TRISC  =  0x87

//Variables accesibles

//Variables accesibles
static int8 SEN_LIGHT_P;
static int8 SEN_LINE_R;
static int8 SEN_LINE_L;
static int8 SEN_OBS_R;
static int8 SEN_OBS_L;


//*****************************************************
//*                  SEN_CONFIG()                     *
//*****************************************************
//*Descripcin: La funcin configura las E/S del PIC y*
//*a su vez configura el ADC                          *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida:                               *
//*****************************************************
void SEN_CONFIG()
{
    //Configuracin PORT_A
    bit_set(LIGHT_SEN_TRIS);
    bit_set(IR_RX_R_TRIS);
    bit_set(LINE_RX_R_TRIS);
    bit_set(IR_RX_L_TRIS);
    bit_clear(LED_TOP_GREEN_TRIS);
    bit_set(LINE_RX_L_TRIS);

    //Configuracin PORT_B
    bit_clear(LINE_TX_TRIS);
    bit_clear(IR_TX_TRIS);
    bit_clear(LED_R_TRIS);
    bit_clear(LED_TOP_RED_TRIS);

    //Configuracin PORT_C
    bit_clear(LED_L_TRIS);

    //Configuracin del ADC
    setup_adc(ADC_CLOCK_DIV_16);
    setup_adc_ports(NO_ANALOGS);

    //No necesita resistencias de pull-up en el
    //PORTB
    port_b_pullups (false);

    //Por defecto el LED verde est apagado
    output_high(LED_TOP_GREEN);
    //Por defecto el LED rojo est apagado
    output_high(LED_TOP_RED);
    //Por defecto el LED izquierdo est apagado
    output_low(LED_L);
    //Por defecto el LED derecho est apagado
    output_low(LED_R);

    //Por defecto el LED infrarrojo est apagado
    output_low(IR_TX);

}
//*****************************************************






//*****************************************************
//*                  SEN_LIGHT()                      *
//*****************************************************
//*Descripcin:Devuelve el porcentage de luz          *
//*recibida por el sensor de luz (0-100)              *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida:                               *
//*****************************************************

void SEN_LIGHT(){

   //Se selecciona el canal0 del ADC y se espera el
   //tiempo de adquisicin del convertidor analgico
   //digital
   setup_adc_ports(ALL_ANALOG);
   set_adc_channel(0);
   Delay_us(100);

   //Lectura y calculo del porcentage de luz incidente
   SEN_LIGHT_P=(read_adc()/2);
   setup_adc_ports(NO_ANALOGS);

}
//*****************************************************





//*****************************************************
//*                 SEN_OBS_DIG()                     *
//*****************************************************
//*Descripcin: Devuelve si se ha detectado un        *
//*obstculo en la izquierda o derecha del robot      *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida: -SEN_OBS_R                    *
//*                     -SEN_OBS_L                    *
//*Salida:              -0:Detectado seal externa    *
//                        (Moway o interferecia)      *
//                      -1:Lectura correcta           *
//*****************************************************

int1 SEN_OBS_DIG(){

   //Comprobacin de la no existencia de seal externa
   //indeseada
   if(input(IR_RX_R)!=0 || input(IR_RX_L)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   return(0);
   }

   //Se manda pulso de luz infrarroja para
   //la detecin de obstculo
   output_high(IR_TX);
   Delay_us(65);

   //Comprobacin si el pulso a rebotado en
   //algn objeto
   SEN_OBS_R=input(IR_RX_R);
   SEN_OBS_L=input(IR_RX_L);

   //Finalizacin del pulso de luz infrarroja
   output_low(IR_TX);
   Delay_us(30);


   //Comprobacin si se ha leido el pulso mandado
   //o interferencia externa
   if(input(IR_RX_R)!=0 || input(IR_RX_L)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   Delay_us(1500);
   return(0);
   }
   Delay_us(1500);
   return(1);
}
//*****************************************************






//*****************************************************
//*               SEN_OBS_ANALOG()                    *
//*****************************************************
//*Descripcin: Devuelve si se ha detectado un        *
//*obstculo en la izquierda o derecha del robot en   *
//*valor proporcional a la distancia.                 *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida: -SEN_OBS_R                    *
//*                     -SEN_OBS_L                    *
//*Salida:              -0:Detectado seal externa    *
//                        (Moway o interferecia)      *
//                      -1:Lectura correcta           *
//*****************************************************
int1 SEN_OBS_ANALOG(){

   //Comprobacin de la no existencia de seal externa
   //indeseada en la derecha
   if(input(IR_RX_R)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   return(0);
   }

   //Se configura los pines del PORTA como analgicos
   //y se elige el canal 1. Se espera el tiempo de
   //adquisicin que en este caso es corto por la
   //resistencia de salida del sensor IR
   setup_adc_ports(ALL_ANALOG);
   set_adc_channel(1);

   //Se manda pulso de luz infrarroja para
   //la detecin de obstculo en la derecha
   output_high(IR_TX);
   Delay_us(65);

   //Lectura del valor del sensor derecho
   SEN_OBS_R=read_adc();

   //Finalizacin del pulso de luz infrarroja
   output_low(IR_TX);
   Delay_us(30);
   setup_adc_ports(NO_ANALOGS);

   //Comprobacin si se ha leido el pulso mandado
   //o interferencia externa
   if(input(IR_RX_R)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   Delay_us(1500);
   return(0);
   }

   Delay_us(1500);


   //Comprobacin de la no existencia de seal externa
   //indeseada en la izquierda
   if(input(IR_RX_L)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   return(0);
   }

    //Se configura los pines del PORTA como analgicos
   //y se elige el canal 1. Se espera el tiempo de
   //adquisicin que en este caso es corto por la
   //resistencia de salida del sensor IR
   setup_adc_ports(ALL_ANALOG);
   set_adc_channel(3);


   //Se manda pulso de luz infrarroja para
   //la detecin de obstculo en la izquierda
   output_high(IR_TX);
   Delay_us(65);


   //Lectura del valor del sensor izquierdo
   SEN_OBS_L=read_adc();

   //Finalizacin del pulso de luz infrarroja
   output_low(IR_TX);
   Delay_us(30);
   setup_adc_ports(NO_ANALOGS);

   //Comprobacin si se ha leido el pulso mandado
   //o interferencia externa
   if(input(IR_RX_L)!=0){
   SEN_OBS_R=0;
   SEN_OBS_L=0;
   Delay_us(1500);
   return(0);
   }
   Delay_us(1500);
   return(1);
}
//*****************************************************







//*****************************************************
//*               SEN_LINE_DIG()                      *
//*****************************************************
//*Descripcin: Devuelve si se ha detectado una       *
//*lnea negra sobre suelo claro                      *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida: -SEN_LINE_R                   *
//*                     -SEN_LINE_L                   *
//*****************************************************
void SEN_LINE_DIG(){

   //Se manda pulso de luz infrarroja para
   //la detecin de lnea
   output_high(LINE_TX);
   Delay_us(900);

   //Lectura de la luz infrarroja rebotada en el suelo
   SEN_LINE_R=input(LINE_RX_R);
   SEN_LINE_L=input(LINE_RX_L);

   //Finalizacin del pulso de luz infrarroja
   output_low(LINE_TX);
}
//*****************************************************








//*****************************************************
//*               SEN_LINE_ANALOG()                   *
//*****************************************************
//*Descripcin: Devuelve el valor de la claridad del  *
//*suelo. Se puede distinguir colores dependiendo del *
//*material.                                          *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida: -SEN_LINE_R                   *
//*                     -SEN_LINE_L                   *
//*****************************************************
void SEN_LINE_ANALOG(){

   //Configuramos las entradas como analgicas
   setup_adc_ports(ALL_ANALOG);

   //Se manda pulso de luz infrarroja para
   //la detecin de lnea
   set_adc_channel(2);
   output_high(LINE_TX);
   Delay_us(900);

   //Lectura del sensor derecho
   SEN_LINE_R=read_adc();

   //Cambio de canal del ADC
   set_adc_channel(4);
   Delay_us(100);

   //Lectura del sensor izquierdo
   SEN_LINE_L=read_adc();

   //Finalizacin del pulso de luz infrarroja
   output_low(LINE_TX);
   setup_adc_ports(NO_ANALOGS);
}
//*****************************************************








//*****************************************************
//*                   LED_xx_xx()                     *
//*****************************************************
//*Descripcin: Funciones de manejo de LED del robot  *
//*****************************************************
//*Variables de entrada:                              *
//*Variables de salida:                               *
//*****************************************************

//Encendido de led delatero derecho
void LED_R_ON(){
   output_high(LED_R);
}
//Apagado de led delatero derecho
void LED_R_OFF(){
   output_low(LED_R);
}
//Parpadeo de led delantero derecho
void LED_R_ON_OFF(){
   output_high(LED_R);
   Delay_ms(50);
   output_low(LED_R);
   Delay_ms(50);
}
//Encendido de led delantero izquierdo
void LED_L_ON(){
   output_high(LED_L);
}
//Apagado de led derantero izquierdo
void LED_L_OFF(){
   output_low(LED_L);
}
//Parpadeo de led delantero izquierdo
void LED_L_ON_OFF(){
   output_high(LED_L);
   Delay_ms(50);
   output_low(LED_L);
   Delay_ms(50);
}
//Encendido de led superior rojo
void LED_TOP_RED_ON(){
   output_low(LED_TOP_RED);
}
//Apagado de led superior rojo
void LED_TOP_RED_OFF(){
   output_high(LED_TOP_RED);
}
//Parpadeo de led superior rojo
void LED_TOP_RED_ON_OFF(){
   output_low(LED_TOP_RED);
   Delay_ms(50);
   output_high(LED_TOP_RED);
   Delay_ms(50);
}
//Encendido de led superior verde
void LED_TOP_GREEN_ON(){
   output_low(LED_TOP_GREEN);
}
//Apagado de led superior verde
void LED_TOP_GREEN_OFF(){
   output_high(LED_TOP_GREEN);
}
//Parpadeo de led superior verde
void LED_TOP_GREEN_ON_OFF(){
   output_low(LED_TOP_GREEN);
   Delay_ms(50);
   output_high(LED_TOP_GREEN);
   Delay_ms(50);
}
//*****************************************************




